@@ -1555,10 +1555,10 @@ fprintf(stdout, "%s %s (C) %s ZeroBeat\n"
 	"-h        : show this help\n"
 	"-v        : show version\n"
 	"\n"
-	"--pmkid-eapol=<file> : input PMKID EAPOL combi hash file\n"
-	"--pmkid=<file>       : input PMKID hash file\n"
-	"--hccapx=<file>      : input hashcat hccapx file\n"
-	"--hccap=<file>       : input hashcat hccap file\n"
+	"--pmkid-eapol=<file> : input PMKID EAPOL (22000) combi hash file\n"
+	"--pmkid=<file>       : input deprecated PMKID (16800) hash file\n"
+	"--hccapx=<file>      : input deprecated hccapx (2500) hash file\n"
+	"--hccap=<file>       : input ancient hccap (2500) file\n"
 	"--john=<file>        : input John the Ripper WPAPSK hash file\n"
 	"--help               : show this help\n"
 	"--version            : show version\n"
