@@ -95,8 +95,9 @@ static bool flagfilterouiclient;
 static uint8_t filterouiclient[3];
 
 static bool flagfilterauthorized;
-static bool flagfilternotauthorized;
+static bool flagfilterchallenge;
 static bool flagfilterrcchecked;
+static bool flagfilterrcnotchecked;
 static bool flagfilterapless;
 
 static int pskptrlen;
@@ -152,59 +153,60 @@ static void printstatus()
 {
 static char *vendor;
 
-printf("\nOUI information file..........: %s\n", usedoui);
-if(ouicount > 0)		printf("OUI entires...................: %d\n", ouicount);
-if(readcount > 0)		printf("total lines read..............: %ld\n", readcount);
+fprintf(stdout, "\nOUI information file..........: %s\n", usedoui);
+if(ouicount > 0)		fprintf(stdout, "OUI entires...................: %d\n", ouicount);
+if(readcount > 0)		fprintf(stdout, "total lines read..............: %ld\n", readcount);
 if(flagvendorout == true)
 	{
-	printf("\n");
+	fprintf(stdout, "\n");
 	return;
 	}
-if(pbkdf2count > 0)			printf("PBKDF2 lines..................: %ld\n", pbkdf2count);
-if(pbkdf2readerrorcount > 0)		printf("PBKDF2 errors.................: %ld\n", pbkdf2readerrorcount);
-if(readerrorcount > 0)			printf("read errors...................: %ld\n", readerrorcount);
-if(pmkideapolcount > 0)			printf("valid hash lines..............: %ld\n", pmkideapolcount);
-if(pmkidcount > 0)			printf("PMKID hash lines..............: %ld\n", pmkidcount);
-if(eapolcount > 0)			printf("EAPOL hash lines..............: %ld\n", eapolcount);
-if(essidlenmin != 0)			printf("filter by ESSID len min.......: %d\n", essidlenmin);
-if(essidlenmax != 32)			printf("filter by ESSID len max.......: %d\n", essidlenmax);
-if(filteressidptr != NULL)		printf("filter by ESSID...............: %s\n", filteressidptr);
-if(filteressidpartptr != NULL)		printf("filter by part of ESSID.......: %s\n", filteressidpartptr);
+if(pbkdf2count > 0)			fprintf(stdout, "PBKDF2 lines..................: %ld\n", pbkdf2count);
+if(pbkdf2readerrorcount > 0)		fprintf(stdout, "PBKDF2 errors.................: %ld\n", pbkdf2readerrorcount);
+if(readerrorcount > 0)			fprintf(stdout, "read errors...................: %ld\n", readerrorcount);
+if(pmkideapolcount > 0)			fprintf(stdout, "valid hash lines..............: %ld\n", pmkideapolcount);
+if(pmkidcount > 0)			fprintf(stdout, "PMKID hash lines..............: %ld\n", pmkidcount);
+if(eapolcount > 0)			fprintf(stdout, "EAPOL hash lines..............: %ld\n", eapolcount);
+if(essidlenmin != 0)			fprintf(stdout, "filter by ESSID len min.......: %d\n", essidlenmin);
+if(essidlenmax != 32)			fprintf(stdout, "filter by ESSID len max.......: %d\n", essidlenmax);
+if(filteressidptr != NULL)		fprintf(stdout, "filter by ESSID...............: %s\n", filteressidptr);
+if(filteressidpartptr != NULL)		fprintf(stdout, "filter by part of ESSID.......: %s\n", filteressidpartptr);
 if(flagfiltermacap == true)
 	{
 	vendor = getvendor(filtermacap);
-	printf("filter by MAC.................: %02x%02x%02x%02x%02x%02x (%s)\n", filtermacap[0], filtermacap[1], filtermacap[2], filtermacap[3], filtermacap[4], filtermacap[5], vendor);
+	fprintf(stdout, "filter by MAC.................: %02x%02x%02x%02x%02x%02x (%s)\n", filtermacap[0], filtermacap[1], filtermacap[2], filtermacap[3], filtermacap[4], filtermacap[5], vendor);
 	}
 if(flagfiltermacclient == true)
 	{
 	vendor = getvendor(filtermacclient);
-	printf("filter by MAC.................: %02x%02x%02x%02x%02x%02x (%s)\n", filtermacclient[0], filtermacclient[1], filtermacclient[2], filtermacclient[3], filtermacclient[4], filtermacclient[5], vendor);
+	fprintf(stdout, "filter by MAC.................: %02x%02x%02x%02x%02x%02x (%s)\n", filtermacclient[0], filtermacclient[1], filtermacclient[2], filtermacclient[3], filtermacclient[4], filtermacclient[5], vendor);
 	}
 
 if(flagfilterouiap == true)
 	{
 	vendor = getvendor(filterouiap);
-	printf("filter AP by OUI..............: %02x%02x%02x (%s)\n", filterouiap[0], filterouiap[1], filterouiap[2], vendor);
+	fprintf(stdout, "filter AP by OUI..............: %02x%02x%02x (%s)\n", filterouiap[0], filterouiap[1], filterouiap[2], vendor);
 	}
-if(filtervendorptr != NULL)		printf("filter AP and CLIENT by VENDOR: %s\n", filtervendorptr);
-if(filtervendorapptr != NULL)		printf("filter AP by VENDOR...........: %s\n", filtervendorapptr);
-if(filtervendorclientptr != NULL)	printf("filter CLIENT by VENDOR.......: %s\n", filtervendorclientptr);
+if(filtervendorptr != NULL)		fprintf(stdout, "filter AP and CLIENT by VENDOR: %s\n", filtervendorptr);
+if(filtervendorapptr != NULL)		fprintf(stdout, "filter AP by VENDOR...........: %s\n", filtervendorapptr);
+if(filtervendorclientptr != NULL)	fprintf(stdout, "filter CLIENT by VENDOR.......: %s\n", filtervendorclientptr);
 if(flagfilterouiclient == true)
 	{
 	vendor = getvendor(filterouiclient);
-	printf("filter CLIENT by OUI..........: %02x%02x%02x (%s)\n", filterouiclient[0], filterouiclient[1], filterouiclient[2], vendor);
+	fprintf(stdout, "filter CLIENT by OUI..........: %02x%02x%02x (%s)\n", filterouiclient[0], filterouiclient[1], filterouiclient[2], vendor);
 	}
-if(flagfilterapless == true)		printf("filter by M2..................: requested from client (AP-LESS)\n");
-if(flagfilterrcchecked == true)		printf("filter by replaycount.........: checked\n");
-if(flagfilterauthorized == true)	printf("filter by status..............: authorized (M1M4, M2M3 or M3M4)\n");
-if(flagfilternotauthorized == true)	printf("filter by status..............: challenge (M1M2)\n");
-if(pmkidwrittencount > 0)		printf("PMKID written.................: %ld\n", pmkidwrittencount);
-if(eapolwrittencount > 0)		printf("EAPOL written.................: %ld\n", eapolwrittencount);
-if(hccapxwrittencount > 0)		printf("EAPOL written to hccapx.......: %ld\n", hccapxwrittencount);
-if(hccapwrittencount > 0)		printf("EAPOL written to hccap........: %ld\n", hccapwrittencount);
-if(johnwrittencount > 0)		printf("EAPOL written to john.........: %ld\n", johnwrittencount);
-if(essidwrittencount > 0)		printf("ESSID (unique) written........: %ld\n", essidwrittencount);
-printf("\n");
+if(flagfilterapless == true)		fprintf(stdout, "filter by M2..................: requested from client (AP-LESS)\n");
+if(flagfilterrcchecked == true)		fprintf(stdout, "filter by replaycount.........: checked\n");
+if(flagfilterrcnotchecked == true)	fprintf(stdout, "filter by replaycount.........: not checked\n");
+if(flagfilterauthorized == true)	fprintf(stdout, "filter by status..............: authorized (M1M4, M2M3 or M3M4)\n");
+if(flagfilterchallenge == true)	fprintf(stdout, "filter by status..............: challenge (M1M2)\n");
+if(pmkidwrittencount > 0)		fprintf(stdout, "PMKID written.................: %ld\n", pmkidwrittencount);
+if(eapolwrittencount > 0)		fprintf(stdout, "EAPOL written.................: %ld\n", eapolwrittencount);
+if(hccapxwrittencount > 0)		fprintf(stdout, "EAPOL written to hccapx.......: %ld\n", hccapxwrittencount);
+if(hccapwrittencount > 0)		fprintf(stdout, "EAPOL written to hccap........: %ld\n", hccapwrittencount);
+if(johnwrittencount > 0)		fprintf(stdout, "EAPOL written to john.........: %ld\n", johnwrittencount);
+if(essidwrittencount > 0)		fprintf(stdout, "ESSID (unique) written........: %ld\n", essidwrittencount);
+fprintf(stdout, "\n");
 return;
 }
 /*===========================================================================*/
@@ -794,9 +796,10 @@ if((filtervendorptr != NULL) || (filtervendorapptr != NULL) || (filtervendorclie
 	if(isoui(zeiger->ap, zeiger->client) == false) return;
 	}
 if((flagfilterapless == true) && ((zeiger->mp &0x10) != 0x10)) return;
-if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) != 0x80)) return;
+if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) == 0x80)) return;
+if((flagfilterrcnotchecked == true) && ((zeiger->mp &0x80) != 0x80)) return;
 if((flagfilterauthorized == true) && ((zeiger->mp &0x07) == 0x00)) return;
-if((flagfilternotauthorized == true) && ((zeiger->mp &0x07) != 0x01)) return;
+if((flagfilterchallenge == true) && ((zeiger->mp &0x07) != 0x01)) return;
 
 wpak = (wpakey_t*)(zeiger->eapol +EAPAUTH_SIZE);
 memset(&hccap, 0, sizeof(hccap_t));
@@ -840,7 +843,7 @@ if(johnoutname != NULL)
 	{
 	if((fh_john = fopen(johnoutname, "a")) == NULL)
 		{
-		printf("error opening file %s: %s\n", johnoutname, strerror(errno));
+		fprintf(stdout, "error opening file %s: %s\n", johnoutname, strerror(errno));
 		return;
 		}
 	}
@@ -897,9 +900,10 @@ if((filtervendorptr != NULL) || (filtervendorapptr != NULL) || (filtervendorclie
 	if(isoui(zeiger->ap, zeiger->client) == false) return;
 	}
 if((flagfilterapless == true) && ((zeiger->mp &0x10) != 0x10)) return;
-if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) != 0x80)) return;
+if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) == 0x80)) return;
+if((flagfilterrcnotchecked == true) && ((zeiger->mp &0x80) != 0x80)) return;
 if((flagfilterauthorized == true) && ((zeiger->mp &0x07) == 0x00)) return;
-if((flagfilternotauthorized == true) && ((zeiger->mp &0x07) != 0x01)) return;
+if((flagfilterchallenge == true) && ((zeiger->mp &0x07) != 0x01)) return;
 
 wpak = (wpakey_t*)(zeiger->eapol +EAPAUTH_SIZE);
 memset(&hccap, 0, sizeof(hccap_t));
@@ -959,7 +963,7 @@ if(hccapoutname != NULL)
 	{
 	if((fh_hccap = fopen(hccapoutname, "a")) == NULL)
 		{
-		printf("error opening file %s: %s\n", hccapoutname, strerror(errno));
+		fprintf(stdout, "error opening file %s: %s\n", hccapoutname, strerror(errno));
 		return;
 		}
 	}
@@ -1022,9 +1026,10 @@ if((filtervendorptr != NULL) || (filtervendorapptr != NULL) || (filtervendorclie
 	if(isoui(zeiger->ap, zeiger->client) == false) return;
 	}
 if((flagfilterapless == true) && ((zeiger->mp &0x10) != 0x10)) return;
-if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) != 0x80)) return;
+if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) == 0x80)) return;
+if((flagfilterrcnotchecked == true) && ((zeiger->mp &0x80) != 0x80)) return;
 if((flagfilterauthorized == true) && ((zeiger->mp &0x07) == 0x00)) return;
-if((flagfilternotauthorized == true) && ((zeiger->mp &0x07) != 0x01)) return;
+if((flagfilterchallenge == true) && ((zeiger->mp &0x07) != 0x01)) return;
 
 wpak = (wpakey_t*)(zeiger->eapol +EAPAUTH_SIZE);
 memset (&hccapx, 0, sizeof(hccapx_t));
@@ -1061,7 +1066,7 @@ if(hccapxoutname != NULL)
 	{
 	if((fh_hccapx = fopen(hccapxoutname, "a")) == NULL)
 		{
-		printf("error opening file %s: %s\n", hccapxoutname, strerror(errno));
+		fprintf(stdout, "error opening file %s: %s\n", hccapxoutname, strerror(errno));
 		return;
 		}
 	}
@@ -1086,7 +1091,7 @@ static struct stat statinfo;
 
 if((fh_essid = fopen(essidoutname, "a")) == NULL)
 	{
-	printf("error opening file %s: %s\n", essidoutname, strerror(errno));
+	fprintf(stdout, "error opening file %s: %s\n", essidoutname, strerror(errno));
 	return;
 	}
 zeigerold = NULL;
@@ -1134,9 +1139,10 @@ if((filtervendorptr != NULL) || (filtervendorapptr != NULL) || (filtervendorclie
 	if(isoui(zeiger->ap, zeiger->client) == false) return;
 	}
 if((flagfilterapless == true) && ((zeiger->mp &0x10) != 0x10)) return;
-if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) != 0x00)) return;
+if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) == 0x80)) return;
+if((flagfilterrcnotchecked == true) && ((zeiger->mp &0x80) != 0x80)) return;
 if((flagfilterauthorized == true) && ((zeiger->mp &0x07) == 0x00)) return;
-if((flagfilternotauthorized == true) && ((zeiger->mp &0x07) != 0x01)) return;
+if((flagfilterchallenge == true) && ((zeiger->mp &0x07) != 0x01)) return;
 if(zeiger->type == HCX_TYPE_PMKID)
 	{
 	fprintf(fh_pmkideapol, "WPA*%02d*%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x*%02x%02x%02x%02x%02x%02x*%02x%02x%02x%02x%02x%02x*",
@@ -1202,7 +1208,7 @@ for(zeiger = hashlist; zeiger < hashlist +pmkideapolcount; zeiger++)
 	strcat(&groupoutname[ceo], ".22000");
 	if((fh_pmkideapol = fopen(groupoutname, "a")) == NULL)
 		{
-		printf("error opening file %s: %s\n", groupoutname, strerror(errno));
+		fprintf(stdout, "error opening file %s: %s\n", groupoutname, strerror(errno));
 		return;
 		}
 	writepmkideapolhashline(fh_pmkideapol, zeiger);
@@ -1232,7 +1238,7 @@ for(zeiger = hashlist; zeiger < hashlist +pmkideapolcount; zeiger++)
 	snprintf(groupoutname, PATH_MAX -1, "%02x%02x%02x.22000", zeiger->ap[0], zeiger->ap[1], zeiger->ap[2]);
 	if((fh_pmkideapol = fopen(groupoutname, "a")) == NULL)
 		{
-		printf("error opening file %s: %s\n", groupoutname, strerror(errno));
+		fprintf(stdout, "error opening file %s: %s\n", groupoutname, strerror(errno));
 		return;
 		}
 	writepmkideapolhashline(fh_pmkideapol, zeiger);
@@ -1262,7 +1268,7 @@ for(zeiger = hashlist; zeiger < hashlist +pmkideapolcount; zeiger++)
 	snprintf(groupoutname, PATH_MAX -1, "%02x%02x%02x%02x%02x%02x.22000", zeiger->ap[0], zeiger->ap[1], zeiger->ap[2], zeiger->ap[3], zeiger->ap[4], zeiger->ap[5]);
 	if((fh_pmkideapol = fopen(groupoutname, "a")) == NULL)
 		{
-		printf("error opening file %s: %s\n", groupoutname, strerror(errno));
+		fprintf(stdout, "error opening file %s: %s\n", groupoutname, strerror(errno));
 		return;
 		}
 	writepmkideapolhashline(fh_pmkideapol, zeiger);
@@ -1292,7 +1298,7 @@ for(zeiger = hashlist; zeiger < hashlist +pmkideapolcount; zeiger++)
 	snprintf(groupoutname, PATH_MAX -1, "%02x%02x%02x%02x%02x%02x.22000", zeiger->client[0], zeiger->client[1], zeiger->client[2], zeiger->client[3], zeiger->client[4], zeiger->client[5]);
 	if((fh_pmkideapol = fopen(groupoutname, "a")) == NULL)
 		{
-		printf("error opening file %s: %s\n", groupoutname, strerror(errno));
+		fprintf(stdout, "error opening file %s: %s\n", groupoutname, strerror(errno));
 		return;
 		}
 	writepmkideapolhashline(fh_pmkideapol, zeiger);
@@ -1325,7 +1331,7 @@ if(pmkideapoloutname != NULL)
 	{
 	if((fh_pmkideapol = fopen(pmkideapoloutname, "a")) == NULL)
 		{
-		printf("error opening file %s: %s\n", pmkideapoloutname, strerror(errno));
+		fprintf(stdout, "error opening file %s: %s\n", pmkideapoloutname, strerror(errno));
 		return;
 		}
 	}
@@ -1371,7 +1377,7 @@ if(pmkideapoloutname != NULL)
 	{
 	if((fh_pmkideapol = fopen(pmkideapoloutname, "a")) == NULL)
 		{
-		printf("error opening file %s: %s\n", pmkideapoloutname, strerror(errno));
+		fprintf(stdout, "error opening file %s: %s\n", pmkideapoloutname, strerror(errno));
 		return;
 		}
 	}
@@ -1415,9 +1421,10 @@ if((filtervendorptr != NULL) || (filtervendorapptr != NULL) || (filtervendorclie
 	if(isoui(zeiger->ap, zeiger->client) == false) return;
 	}
 if((flagfilterapless == true) && ((zeiger->mp &0x10) != 0x10)) return;
-if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) != 0x00)) return;
+if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) == 0x80)) return;
+if((flagfilterrcnotchecked == true) && ((zeiger->mp &0x80) != 0x80)) return;
 if((flagfilterauthorized == true) && ((zeiger->mp &0x07) == 0x00)) return;
-if((flagfilternotauthorized == true) && ((zeiger->mp &0x07) != 0x01)) return;
+if((flagfilterchallenge == true) && ((zeiger->mp &0x07) != 0x01)) return;
 
 fprintf(fh_pmkideapol, "SSID.......: %.*s\n", zeiger->essidlen, zeiger->essid);
 vendor = getvendor(zeiger->ap);
@@ -1478,11 +1485,11 @@ if(strcmp(vendorinfooutname, "stdout") != 0)
 	{
 	if((fh_info = fopen(vendorinfooutname, "a")) == NULL)
 		{
-		printf("error opening file %s: %s\n", vendorinfooutname, strerror(errno));
+		fprintf(stdout, "error opening file %s: %s\n", vendorinfooutname, strerror(errno));
 		return;
 		}
 	vendor = getvendor(hashlist->ap);
-	fprintf(stdout, "%02x%02x%02x\t%s\t[ACCESS POINT]\n", hashlist->ap[0], hashlist->ap[1], hashlist->ap[2], vendor);
+	fprintf(fh_info, "%02x%02x%02x\t%s\t[ACCESS POINT]\n", hashlist->ap[0], hashlist->ap[1], hashlist->ap[2], vendor);
 	for(zeiger = hashlist +1; zeiger < hashlist +pmkideapolcount; zeiger++)
 		{
 		if(memcmp((zeiger -1)->ap, zeiger->ap, 3) != 0)
@@ -1520,11 +1527,11 @@ if(strcmp(vendorinfooutname, "stdout") != 0)
 	{
 	if((fh_info = fopen(vendorinfooutname, "a")) == NULL)
 		{
-		printf("error opening file %s: %s\n", vendorinfooutname, strerror(errno));
+		fprintf(stdout, "error opening file %s: %s\n", vendorinfooutname, strerror(errno));
 		return;
 		}
 	vendor = getvendor(hashlist->client);
-	fprintf(stdout, "%02x%02x%02x\t%s\t[CLIENT]\n", hashlist->client[0], hashlist->client[1], hashlist->client[2], vendor);
+	fprintf(fh_info, "%02x%02x%02x\t%s\t[CLIENT]\n", hashlist->client[0], hashlist->client[1], hashlist->client[2], vendor);
 	for(zeiger = hashlist +1; zeiger < hashlist +pmkideapolcount; zeiger++)
 		{
 		if(memcmp((zeiger -1)->client, zeiger->client, 3) != 0)
@@ -1560,7 +1567,7 @@ if(strcmp(infooutname, "stdout") != 0)
 	{
 	if((fh_info = fopen(infooutname, "a")) == NULL)
 		{
-		printf("error opening file %s: %s\n", infooutname, strerror(errno));
+		fprintf(stdout, "error opening file %s: %s\n", infooutname, strerror(errno));
 		return;
 		}
 	for(zeiger = hashlist; zeiger < hashlist +pmkideapolcount; zeiger++) writepmkideapolhashlineinfo(fh_info, zeiger);
@@ -1595,8 +1602,8 @@ memset(buff, 0, bufflen);
 p = 0;
 while((lineptr[p] != '*') && (lineptr[p] != 0) && (p /2 <= bufflen))
 	{
-	if(!isxdigit(lineptr[p +0])) return 0;
-	if(!isxdigit(lineptr[p +1])) return 0;
+	if(!isxdigit((unsigned char)lineptr[p +0])) return 0;
+	if(!isxdigit((unsigned char)lineptr[p +1])) return 0;
 	if((lineptr[p +1] == '*') && (lineptr[p +1] == 0)) return 0;
 	idx0 = ((uint8_t)lineptr[p +0] &0x1F) ^0x10;
 	idx1 = ((uint8_t)lineptr[p +1] &0x1F) ^0x10;
@@ -1654,9 +1661,9 @@ static char linein[PMKIDEAPOL_BUFFER_LEN];
 
 maclistskipmax = 1000;
 if((maclistskip = (maclist_t*)calloc(maclistskipmax, MACLIST_SIZE)) == NULL) return;
-if((fh_maclistin = fopen(macskipname, "rb")) == NULL)
+if((fh_maclistin = fopen(macskipname, "r")) == NULL)
 	{
-	printf("error opening file %s: %s\n", macskipname, strerror(errno));
+	fprintf(stdout, "error opening file %s: %s\n", macskipname, strerror(errno));
 	return;
 	}
 
@@ -1670,7 +1677,7 @@ while(1)
 		p2 = 0;
 		for(p1 = 0; p1 < 17; p1++)
 			{
-			if(isxdigit(linein[p1]))
+			if(isxdigit((unsigned char)linein[p1]))
 				{
 				linein[p2] = linein[p1];
 				p2++;
@@ -1688,7 +1695,7 @@ while(1)
 		maclistskipnew = realloc(maclistskip, maclistskipmax *MACLIST_SIZE);
 		if(maclistskipnew == NULL)
 			{
-			printf("failed to allocate memory for internal list\n");
+			fprintf(stderr, "failed to allocate memory for internal list\n");
 			exit(EXIT_FAILURE);
 			}
 		maclistskip = maclistskipnew;
@@ -1760,9 +1767,9 @@ static char linein[PMKIDEAPOL_BUFFER_LEN];
 
 maclistinmax = 1000;
 if((maclistin = (maclist_t*)calloc(maclistinmax, MACLIST_SIZE)) == NULL) return;
-if((fh_maclistin = fopen(maclistinname, "rb")) == NULL)
+if((fh_maclistin = fopen(maclistinname, "r")) == NULL)
 	{
-	printf("error opening file %s: %s\n", maclistinname, strerror(errno));
+	fprintf(stdout, "error opening file %s: %s\n", maclistinname, strerror(errno));
 	return;
 	}
 zeiger = maclistin;
@@ -1775,7 +1782,7 @@ while(1)
 		p2 = 0;
 		for(p1 = 0; p1 < 17; p1++)
 			{
-			if(isxdigit(linein[p1]))
+			if(isxdigit((unsigned char)linein[p1]))
 				{
 				linein[p2] = linein[p1];
 				p2++;
@@ -1793,7 +1800,7 @@ while(1)
 		maclistinnew = realloc(maclistin, maclistinmax *MACLIST_SIZE);
 		if(maclistinnew == NULL)
 			{
-			printf("failed to allocate memory for internal list\n");
+			fprintf(stdout, "failed to allocate memory for internal list\n");
 			exit(EXIT_FAILURE);
 			}
 		maclistin = maclistinnew;
@@ -1807,7 +1814,7 @@ if(pmkideapoloutname != NULL)
 	{
 	if((fh_pmkideapol = fopen(pmkideapoloutname, "a")) == NULL)
 		{
-		printf("error opening file %s: %s\n", pmkideapoloutname, strerror(errno));
+		fprintf(stdout, "error opening file %s: %s\n", pmkideapoloutname, strerror(errno));
 		free(maclistin);
 		return;
 		}
@@ -1866,9 +1873,9 @@ static char linein[PMKIDEAPOL_BUFFER_LEN];
 
 essidlistinmax = 1000;
 if((essidlistin = (essidlist_t*)calloc(essidlistinmax, ESSIDLIST_SIZE)) == NULL) return;
-if((fh_essidlistin = fopen(essidlistinname, "rb")) == NULL)
+if((fh_essidlistin = fopen(essidlistinname, "r")) == NULL)
 	{
-	printf("error opening file %s: %s\n", essidlistinname, strerror(errno));
+	fprintf(stdout, "error opening file %s: %s\n", essidlistinname, strerror(errno));
 	return;
 	}
 
@@ -1897,7 +1904,7 @@ while(1)
 		essidlistinnew = realloc(essidlistin, essidlistinmax *ESSIDLIST_SIZE);
 		if(essidlistinnew == NULL)
 			{
-			printf("failed to allocate memory for internal list\n");
+			fprintf(stderr, "failed to allocate memory for internal list\n");
 			exit(EXIT_FAILURE);
 			}
 		essidlistin = essidlistinnew;
@@ -1914,7 +1921,7 @@ if(pmkideapoloutname == NULL)
 	}
 if((fh_pmkideapol = fopen(pmkideapoloutname, "a")) == NULL)
 	{
-	printf("error opening file %s: %s\n", pmkideapoloutname, strerror(errno));
+	fprintf(stdout, "error opening file %s: %s\n", pmkideapoloutname, strerror(errno));
 	free(essidlistin);
 	return;
 	}
@@ -2060,7 +2067,7 @@ while(1)
 		hashlistnew = realloc(hashlist, hashlistcount *HASHLIST_SIZE);
 		if(hashlistnew == NULL)
 			{
-			printf("failed to allocate memory for internal list\n");
+			fprintf(stderr, "failed to allocate memory for internal list\n");
 			exit(EXIT_FAILURE);
 			}
 		hashlist = hashlistnew;
@@ -2079,7 +2086,7 @@ static char linein[PBKDF2_LINE_LEN +1];
 
 if((fh_pbkdf2 = fopen(pkdf2inname, "r")) == NULL)
 	{
-	printf("error opening file %s: %s\n", pkdf2inname, strerror(errno));
+	fprintf(stdout, "error opening file %s: %s\n", pkdf2inname, strerror(errno));
 	return false;
 	}
 
@@ -2105,8 +2112,7 @@ while(1)
 		continue;
 		}
 	pskpos[0] = 0;
-
-	printf("%s %s\n", &linein[65], pskpos +1);
+	fprintf(stdout, "%s %s\n", &linein[65], pskpos +1);
 	pbkdf2count++;
 	}
 fclose(fh_pbkdf2);
@@ -2130,7 +2136,7 @@ static int ret;
 
 for(c = 7; c < len; c++)
 	{
-	if(islower(linein[c])) linein[c] = toupper(linein[c]);
+	if(islower((unsigned char)linein[c])) linein[c] = toupper((unsigned char)linein[c]);
 	}
 
 ret = 0;
@@ -2200,7 +2206,7 @@ while(1)
 		ouilistnew = realloc(ouilist, ouilistcount *OUILIST_SIZE);
 		if(ouilistnew == NULL)
 			{
-			printf("failed to allocate memory for internal list\n");
+			fprintf(stderr, "failed to allocate memory for internal list\n");
 			exit(EXIT_FAILURE);
 			}
 		ouilist = ouilistnew;
@@ -2238,10 +2244,10 @@ if(stat(ouinameuserpath, &statinfo) == -1)
 		}
 	}
 strncat(ouinameuserpath, ouiname, PATH_MAX -1);
-printf("start downloading oui from http://standards-oui.ieee.org to: %s\n", ouinameuserpath);
+fprintf(stdout, "start downloading oui from http://standards-oui.ieee.org to: %s\n", ouinameuserpath);
 if((fhoui = fopen(ouinameuserpath, "w")) == NULL)
 	{
-	fprintf(stderr, "error creating file %s", ouiname);
+	fprintf(stdout, "error creating file %s", ouiname);
 	return;
 	}
 hnd = curl_easy_init ();
@@ -2254,24 +2260,24 @@ curl_easy_cleanup(hnd);
 fclose(fhoui);
 if(ret != 0)
 	{
-	fprintf(stderr, "download not successful");
+	fprintf(stdout, "download not successful");
 	return;
 	}
-printf("download finished\n");
+fprintf(stdout, "download finished\n");
 return;
 }
 /*===========================================================================*/
 __attribute__ ((noreturn))
 static void version(char *eigenname)
 {
-printf("%s %s (C) %s ZeroBeat\n", eigenname, VERSION_TAG, VERSION_YEAR);
+fprintf(stdout, "%s %s (C) %s ZeroBeat\n", eigenname, VERSION_TAG, VERSION_YEAR);
 exit(EXIT_SUCCESS);
 }
 /*---------------------------------------------------------------------------*/
 __attribute__ ((noreturn))
 static void usage(char *eigenname)
 {
-printf("%s %s (C) %s ZeroBeat\n"
+fprintf(stdout, "%s %s (C) %s ZeroBeat\n"
 	"usage:\n"
 	"%s <options>\n"
 	"\n"
@@ -2326,26 +2332,27 @@ printf("%s %s (C) %s ZeroBeat\n"
 	"--vendor=<VENDOR>            : filter AP or CLIENT by (part of) VENDOR name\n"
 	"--vendor-ap=<VENDOR>         : filter AP by (part of) VENDOR name\n"
 	"--vendor-client=<VENDOR>     : filter CLIENT by (part of) VENDOR name\n"
-	"--authorized                 : filter EAPOL pairs by status authorized\n"
-	"--notauthorized              : filter EAPOL pairs by status CHALLENGE (not authorized)\n"
+	"--authorized                 : filter EAPOL pairs by status authorized (M2M3, M3M4, M1M4)\n"
+	"--challenge                  : filter EAPOL pairs by status CHALLENGE (M1M2, M1M2ROGUE)\n"
 	"--rc                         : filter EAPOL pairs by replaycount status checked\n"
+	"--rc-not                     : filter EAPOL pairs by replaycount status not checked\n"
 	"--apless                     : filter EAPOL pairs by status M1M2ROGUE (M2 requested from CLIENT)\n"
 	"--info=<file>                : output detailed information about content of hash file\n"
-	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
+	"                               no filter options available\n"
 	"--info=stdout                : stdout output detailed information about content of hash file\n"
-	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
+	"                               no filter options available\n"
 	"--info-vendor=<file>         : output detailed information about ACCESS POINT and CLIENT VENDORs\n"
-	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
+	"                               no filter options available\n"
 	"--info-vendor-ap=<file>      : output detailed information about ACCESS POINT VENDORs\n"
-	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
+	"                               no filter options available\n"
 	"--info-vendor-client=<file>  : output detailed information about ACCESS POINT VENDORs\n"
-	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
+	"                               no filter options available\n"
 	"--info-vendor=stdout         : stdout output detailed information about ACCESS POINT and CLIENT VENDORs\n"
-	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
+	"                               no filter options available\n"
 	"--info-vendor-ap=stdout      : stdout output detailed information about ACCESS POINT VENDORs\n"
-	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
+	"                               no filter options available\n"
 	"--info-vendor-client=stdout  : stdout output detailed information about ACCESS POINT VENDORs\n"
-	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
+	"                               no filter options available\n"
 	"--psk=<PSK>                  : pre-shared key to test\n"
 	"                               due to PBKDF2 calculation this is a very slow process\n"
 	"                               no nonce error corrections\n"
@@ -2365,7 +2372,7 @@ exit(EXIT_SUCCESS);
 __attribute__ ((noreturn))
 static void usageerror(char *eigenname)
 {
-printf("%s %s (C) %s by ZeroBeat\n"
+fprintf(stdout, "%s %s (C) %s by ZeroBeat\n"
 	"usage: %s -h for help\n", eigenname, VERSION_TAG, VERSION_YEAR, eigenname);
 exit(EXIT_FAILURE);
 }
@@ -2426,8 +2433,9 @@ static const struct option long_options[] =
 	{"vendor-ap",			required_argument,	NULL,	HCX_FILTER_VENDOR_AP},
 	{"vendor-client",		required_argument,	NULL,	HCX_FILTER_VENDOR_CLIENT},
 	{"rc",				no_argument,		NULL,	HCX_FILTER_RC},
+	{"rc-not",			no_argument,		NULL,	HCX_FILTER_RC_NOT},
 	{"authorized",			no_argument,		NULL,	HCX_FILTER_M12},
-	{"notauthorized",		no_argument,		NULL,	HCX_FILTER_M1234},
+	{"challenge",			no_argument,		NULL,	HCX_FILTER_M1234},
 	{"apless",			no_argument,		NULL,	HCX_FILTER_M1M2ROGUE},
 	{"psk",				required_argument,	NULL,	HCX_PSK},
 	{"pmk",				required_argument,	NULL,	HCX_PMK},
@@ -2477,8 +2485,9 @@ flagfiltermacclient = false;
 flagfilterouiap = false;
 flagfilterouiclient = false;
 flagfilterauthorized = false;
-flagfilternotauthorized = false;
+flagfilterchallenge = false;
 flagfilterrcchecked = false;
+flagfilterrcnotchecked = false;
 flagfilterapless = false;
 flagpsk = false;
 flagpmk = false;
@@ -2626,7 +2635,7 @@ while((auswahl = getopt_long (argc, argv, short_options, long_options, &index))
 		p2 = 0;
 		for(p1 = 0; p1 < l; p1++)
 			{
-			if(isxdigit(optarg[p1]))
+			if(isxdigit((unsigned char)optarg[p1]))
 				{
 				optarg[p2] = optarg[p1];
 				p2++;
@@ -2647,7 +2656,7 @@ while((auswahl = getopt_long (argc, argv, short_options, long_options, &index))
 		p2 = 0;
 		for(p1 = 0; p1 < l; p1++)
 			{
-			if(isxdigit(optarg[p1]))
+			if(isxdigit((unsigned char)optarg[p1]))
 				{
 				optarg[p2] = optarg[p1];
 				p2++;
@@ -2668,7 +2677,7 @@ while((auswahl = getopt_long (argc, argv, short_options, long_options, &index))
 		p2 = 0;
 		for(p1 = 0; p1 < l; p1++)
 			{
-			if(isxdigit(optarg[p1]))
+			if(isxdigit((unsigned char)optarg[p1]))
 				{
 				optarg[p2] = optarg[p1];
 				p2++;
@@ -2697,7 +2706,7 @@ while((auswahl = getopt_long (argc, argv, short_options, long_options, &index))
 		p2 = 0;
 		for(p1 = 0; p1 < l; p1++)
 			{
-			if(isxdigit(optarg[p1]))
+			if(isxdigit((unsigned char)optarg[p1]))
 				{
 				optarg[p2] = optarg[p1];
 				p2++;
@@ -2723,7 +2732,7 @@ while((auswahl = getopt_long (argc, argv, short_options, long_options, &index))
 			}
 		for(p1 = 0; p1 < l; p1++)
 			{
-			if(islower(filtervendorptr[p1])) filtervendorptr[p1] = toupper(filtervendorptr[p1]);
+			if(islower((unsigned char)filtervendorptr[p1])) filtervendorptr[p1] = toupper((unsigned char)filtervendorptr[p1]);
 			}
 		break;
 
@@ -2737,7 +2746,7 @@ while((auswahl = getopt_long (argc, argv, short_options, long_options, &index))
 			}
 		for(p1 = 0; p1 < l; p1++)
 			{
-			if(islower(filtervendorapptr[p1])) filtervendorapptr[p1] = toupper(filtervendorapptr[p1]);
+			if(islower((unsigned char)filtervendorapptr[p1])) filtervendorapptr[p1] = toupper((unsigned char)filtervendorapptr[p1]);
 			}
 		break;
 
@@ -2751,7 +2760,7 @@ while((auswahl = getopt_long (argc, argv, short_options, long_options, &index))
 			}
 		for(p1 = 0; p1 < l; p1++)
 			{
-			if(islower(filtervendorclientptr[p1])) filtervendorclientptr[p1] = toupper(filtervendorclientptr[p1]);
+			if(islower((unsigned char)filtervendorclientptr[p1])) filtervendorclientptr[p1] = toupper((unsigned char)filtervendorclientptr[p1]);
 			}
 		break;
 
@@ -2759,12 +2768,16 @@ while((auswahl = getopt_long (argc, argv, short_options, long_options, &index))
 		flagfilterrcchecked = true;
 		break;
 
+		case HCX_FILTER_RC_NOT:
+		flagfilterrcnotchecked = true;
+		break;
+
 		case HCX_FILTER_M12:
 		flagfilterauthorized = true;
 		break;
 
 		case HCX_FILTER_M1234:
-		flagfilternotauthorized = true;
+		flagfilterchallenge = true;
 		break;
 
 		case HCX_FILTER_M1M2ROGUE:
@@ -2865,45 +2878,76 @@ if(pmkideapolinname != NULL)
 	{
 	if((fh_pmkideapol = fopen(pmkideapolinname, "r")) == NULL)
 		{
-		printf("error opening file %s: %s\n", pmkideapolinname, strerror(errno));
+		fprintf(stdout, "error opening file %s: %s\n", pmkideapolinname, strerror(errno));
 		closelists();
 		exit(EXIT_FAILURE);
 		}
 	}
 
 if(fh_pmkideapol != NULL) readpmkideapolfile(fh_pmkideapol);
-if((pmkideapolcount > 0) && (macskipname != NULL)) removepmkideapol(macskipname);
+
+if(pmkideapolcount == 0)
+	{
+	fprintf(stdout, "no hashes loaded\n");
+	if(fh_pmkideapol != NULL) fclose(fh_pmkideapol);
+	closelists();
+	return EXIT_SUCCESS;
+	}
+if(infooutname != NULL)
+	{
+	writeinfofile(infooutname);
+	if(fh_pmkideapol != NULL) fclose(fh_pmkideapol);
+	closelists();
+	return EXIT_SUCCESS;
+	}
+
+if(infovendoroutname != NULL)
+	{
+	writevendorapinfofile(infovendoroutname);
+	writevendorclientinfofile(infovendoroutname);
+	if(fh_pmkideapol != NULL) fclose(fh_pmkideapol);
+	closelists();
+	return EXIT_SUCCESS;
+	}
+else if(infovendorapoutname != NULL)
+	{
+	writevendorapinfofile(infovendorapoutname);
+	if(fh_pmkideapol != NULL) fclose(fh_pmkideapol);
+	closelists();
+	return EXIT_SUCCESS;
+	}
+else if(infovendorclientoutname != NULL)
+	{
+	writevendorclientinfofile(infovendorclientoutname);
+	if(fh_pmkideapol != NULL) fclose(fh_pmkideapol);
+	closelists();
+	return EXIT_SUCCESS;
+	}
+
+if(macskipname != NULL) removepmkideapol(macskipname);
 
 if(hashtypein > 0) hashtype = hashtypein;
 
-if((pmkideapolcount > 0) && (essidoutname != NULL)) processessid(essidoutname);
-if((pmkideapolcount > 0) && (pmkideapoloutname != NULL) && (essidinname == NULL))
+if(essidoutname != NULL) processessid(essidoutname);
+if((pmkideapoloutname != NULL) && (essidinname == NULL))
 	{
 	if((lcmin == 0) && (lcmax == 0)) writeeapolpmkidfile(pmkideapoloutname);
 	else writelceapolpmkidfile(pmkideapoloutname, lcmin, lcmax);
 	}
-if((pmkideapolcount > 0) && (flagessidgroup == true)) writeeapolpmkidessidgroups();
-if((pmkideapolcount > 0) && (flagmacapgroup == true)) writeeapolpmkidmacapgroups();
-if((pmkideapolcount > 0) && (flagmacclientgroup == true)) writeeapolpmkidmacclientgroups();
-if((pmkideapolcount > 0) && (flagouigroup == true)) writeeapolpmkidouigroups();
-if((pmkideapolcount > 0) && (flagpsk == true)) testhashfilepsk();
-if((pmkideapolcount > 0) && (flagpmk == true)) testhashfilepmk();
-if((pmkideapolcount > 0) && (hccapxoutname != NULL)) writehccapxfile(hccapxoutname);
-if((pmkideapolcount > 0) && (hccapoutname != NULL)) writehccapfile(hccapoutname);
-if((pmkideapolcount > 0) && (flaghccapsingleout == true)) writehccapsinglefile();
-if((pmkideapolcount > 0) && (johnoutname != NULL)) writejohnfile(johnoutname);
-if((pmkideapolcount > 0) && (pmkideapoloutname != NULL) && (essidinname != NULL)) processessidfile(essidinname, pmkideapoloutname);
-if((pmkideapolcount > 0) && (macinname != NULL)) processmacfile(macinname, pmkideapoloutname);
-if((pmkideapolcount > 0) && (infooutname != NULL)) writeinfofile(infooutname);
-if((pmkideapolcount > 0) && (infovendoroutname != NULL))
-	{
-	writevendorapinfofile(infovendoroutname);
-	writevendorclientinfofile(infovendoroutname);
-	}
-else if((pmkideapolcount > 0) && (infovendorapoutname != NULL)) writevendorapinfofile(infovendorapoutname);
-else if((pmkideapolcount > 0) && (infovendorclientoutname != NULL)) writevendorclientinfofile(infovendorclientoutname);
+if(flagessidgroup == true) writeeapolpmkidessidgroups();
+if(flagmacapgroup == true) writeeapolpmkidmacapgroups();
+if(flagmacclientgroup == true) writeeapolpmkidmacclientgroups();
+if(flagouigroup == true) writeeapolpmkidouigroups();
+if(flagpsk == true) testhashfilepsk();
+if(flagpmk == true) testhashfilepmk();
+if(hccapxoutname != NULL) writehccapxfile(hccapxoutname);
+if(hccapoutname != NULL) writehccapfile(hccapoutname);
+if(flaghccapsingleout == true) writehccapsinglefile();
+if(johnoutname != NULL) writejohnfile(johnoutname);
+if((pmkideapoloutname != NULL) && (essidinname != NULL)) processessidfile(essidinname, pmkideapoloutname);
+if(macinname != NULL) processmacfile(macinname, pmkideapoloutname);
 
-if((infooutname == NULL) && (infovendoroutname == NULL) && (infovendorapoutname == NULL) && (infovendorclientoutname == NULL)) printstatus();
+printstatus();
 if(fh_pmkideapol != NULL) fclose(fh_pmkideapol);
 closelists();
 return EXIT_SUCCESS;
