@@ -32,15 +32,15 @@ static char lineout[LINEIN_MAX] = {};
 px = 0;
 for(p = 0; p < len; p++)
 	{
-	if(isupper(linein[p]))
+	if(isupper((unsigned char)linein[p]))
 		{
-		lineout[p] = tolower(linein[p]);
+		lineout[p] = tolower((unsigned char)linein[p]);
 		}
 	else
 		{
 		lineout[p] = linein[p];
 		}
-	if(isxdigit(lineout[p]))
+	if(isxdigit((unsigned char)lineout[p]))
 		{
 		px++;
 		}
@@ -63,15 +63,15 @@ static char lineout[LINEIN_MAX] = {};
 px = 0;
 for(p = 0; p < len; p++)
 	{
-	if(islower(linein[p]))
+	if(islower((unsigned char)linein[p]))
 		{
-		lineout[p] = toupper(linein[p]);
+		lineout[p] = toupper((unsigned char)linein[p]);
 		}
 	else
 		{
 		lineout[p] = linein[p];
 		}
-	if(isxdigit(lineout[p]))
+	if(isxdigit((unsigned char)lineout[p]))
 		{
 		px++;
 		}
@@ -94,15 +94,15 @@ static char lineout[LINEIN_MAX] = {};
 px = 0;
 for(p = 0; p < len; p++)
 	{
-	if(isupper(linein[p]))
+	if(isupper((unsigned char)linein[p]))
 		{
-		lineout[p] = tolower(linein[p]);
+		lineout[p] = tolower((unsigned char)linein[p]);
 		}
 	else
 		{
 		lineout[p] = linein[p];
 		}
-	if(isxdigit(lineout[p]))
+	if(isxdigit((unsigned char)lineout[p]))
 		{
 		px++;
 		}
@@ -111,7 +111,7 @@ lineout[p] = 0;
 
 if((p != px) && ((lineout[0] >= 'a') && (lineout[0] <= 'z')))
 	{
-	lineout[0] = toupper(linein[0]);
+	lineout[0] = toupper((unsigned char)linein[0]);
 	fprintf(fh_out, "%s\n", lineout);
 	}
 return;
@@ -194,15 +194,15 @@ static char lineout[LINEIN_MAX] = {};
 pd = 0;
 for(p = 0; p < len; p++)
 	{
-	if(isupper(linein[p]))
+	if(isupper((unsigned char)linein[p]))
 		{
-		lineout[p] = tolower(linein[p]);
+		lineout[p] = tolower((unsigned char)linein[p]);
 		}
 	else
 		{
 		lineout[p] = linein[p];
 		}
-	if(isdigit(lineout[p]))
+	if(isdigit((unsigned char)lineout[p]))
 		{
 		pd++;
 		}
@@ -225,15 +225,15 @@ static char lineout[LINEIN_MAX] = {};
 pd = 0;
 for(p = 0; p < len; p++)
 	{
-	if(islower(linein[p]))
+	if(islower((unsigned char)linein[p]))
 		{
-		lineout[p] = toupper(linein[p]);
+		lineout[p] = toupper((unsigned char)linein[p]);
 		}
 	else
 		{
 		lineout[p] = linein[p];
 		}
-	if(isdigit(lineout[p]))
+	if(isdigit((unsigned char)lineout[p]))
 		{
 		pd++;
 		}
@@ -254,7 +254,7 @@ static int p, pd;
 pd = 0;
 for(p = 0; p < len; p++)
 	{
-	if(isdigit(linein[p]))
+	if(isdigit((unsigned char)linein[p]))
 		{
 		pd++;
 		}
@@ -303,7 +303,7 @@ static char lineout[LINEIN_MAX] = {};
 o = 0;
 for(i = 0; i < len; i++)
 	{
-	if(isxdigit(linein[i]))
+	if(isxdigit((unsigned char)linein[i]))
 		{
 		lineout[o] = linein[i];
 		o++;
@@ -363,7 +363,7 @@ static char lineout[LINEIN_MAX] = {};
 o = 0;
 for(i = 0; i < len; i++)
 	{
-	if(isdigit(linein[i]))
+	if(isdigit((unsigned char)linein[i]))
 		{
 		lineout[o] = linein[i];
 		o++;
@@ -612,19 +612,19 @@ return;
 __attribute__ ((noreturn))
 static void version(char *eigenname)
 {
-printf("%s %s (C) %s ZeroBeat\n", eigenname, VERSION_TAG, VERSION_YEAR);
+fprintf(stdout, "%s %s (C) %s ZeroBeat\n", eigenname, VERSION_TAG, VERSION_YEAR);
 exit(EXIT_SUCCESS);
 }
 /*---------------------------------------------------------------------------*/
 __attribute__ ((noreturn))
 static void usage(char *eigenname)
 {
-printf("%s %s (C) %s ZeroBeat\n"
+fprintf(stdout, "%s %s (C) %s ZeroBeat\n"
 	"usage:\n"
 	"%s <options>\n"
 	"\n"
 	"options:\n"
-	"-i        : input wordlist\n"
+	"-i <file> : input wordlist\n"
 	"-o <file> : output wordlist to file\n"
 	"-h        : show this help\n"
 	"-v        : show version\n"
@@ -640,10 +640,10 @@ printf("%s %s (C) %s ZeroBeat\n"
 	"--version        : show version\n"
 	"\n"
 	"examples:\n"
-	"hcxwltool -i wordlist --straight | sort | uniq |  | sort | uniq | hashcat -m 2500 hashfile.hccapx\n"
-	"hcxwltool -i wordlist --digit --length=10 | sort | uniq |  | sort | uniq | hashcat -m 2500 hashfile.hccapx\n"
-	"hcxwltool -i wordlist --digit | sort | uniq | hashcat -m 16800 hashfile.16800\n"
-	"hcxwltool -i wordlist --xdigit | sort | uniq | john --stdin --format=wpapsk-opencl hashfile.16800\n"
+	"hcxwltool -i wordlist --straight | sort | uniq |  | sort | uniq | hashcat -m 22000 hashfile.hc22000\n"
+	"hcxwltool -i wordlist --digit --length=10 | sort | uniq |  | sort | uniq | hashcat -m 22000 hashfile.hc22000\n"
+	"hcxwltool -i wordlist --digit | sort | uniq | hashcat -m 22000 hashfile.hc22000\n"
+	"hcxwltool -i wordlist --xdigit | sort | uniq | john --stdin --format=wpapsk-opencl john.hashfile\n"
 	"\n", eigenname, VERSION_TAG, VERSION_YEAR, eigenname);
 exit(EXIT_SUCCESS);
 }
@@ -651,7 +651,7 @@ exit(EXIT_SUCCESS);
 __attribute__ ((noreturn))
 static void usageerror(char *eigenname)
 {
-printf("%s %s (C) %s by ZeroBeat\n"
+fprintf(stdout, "%s %s (C) %s by ZeroBeat\n"
 	"usage: %s -h for help\n", eigenname, VERSION_TAG, VERSION_YEAR, eigenname);
 exit(EXIT_FAILURE);
 }
